/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Pearltrees add-on AMO, Copyright(C), 2009, Broceliand SAS, Paris, France 
 * (company in charge of developing Pearltrees)
 * 
 * This file is part of “Pearltrees add-on AMO”.  
 * 
 * Pearltrees add-on AMO is free software: you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License version 3 as published by the Free Software Foundation.
 * 
 * Pearltrees add-on AMO is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with Pearltrees add-on AMO. 
 * If not, see <http://www.gnu.org/licenses/>
 * 
 * ***** END LICENSE BLOCK ***** */

// ///////////////////////////////////////////////////////////////////////////////
// Record button controller
// ///////////////////////////////////////////////////////////////////////////////

var BRO_recordButtonController = {
	
	_recordButtonMenuItemCommandFired:false,
		
	/**
	 * Start / Stop recording
	 * 
	 * @param event
	 */
	onClickRecordButton : function(event) {	
		if(BRO_toolbar.showHelpOnRecording) {
			BRO_toolbar.showHelpRecording();
	    	BRO_toolbar.setFirstUse(false);
		}
	
		if(this._recordButtonMenuItemCommandFired) {
			this._recordButtonMenuItemCommandFired = false;
			return;  
		}
		
		// Don't allow multiple record on the same URL, just play the effect
		if (BRO_toolbar.recordingMode == RECORING_MODE_ONE_BY_ONE && BRO_toolbar.isCurrentUrlRecorded()) {
			// play fake recording effect
			BRO_ButtonsHandler.startRecording();
			BRO_recordButtonController.refreshRecordButtonLabel(true);
			BRO_buttonEffectHelper.runIsRecordingEffect();
			BRO_tools.callWithDelay('BRO_ButtonsHandler.stopRecording()', BRO_buttonEffectHelper.START_RECORDING_EFFECT_TIME);
			return;
		}

		BRO_toolbar.onUseToolbar();

		if (BRO_toolbar.isRecording) { 
			BRO_toolbar.setRecording(false);
		} else {
			var url = BRO_browserManager.getSelectedBrowserUrl();
			if (BRO_toolbar.recordingMode == RECORING_MODE_ONE_BY_ONE && BRO_model.isValidUrl(url)) {
				BRO_toolbar.setRecording(true);
				BRO_ButtonsHandler.recordCurrentPage();
				BRO_tools.callWithDelay('BRO_toolbar.setRecording(false)', BRO_buttonEffectHelper.START_RECORDING_EFFECT_TIME);
			} else if (BRO_toolbar.recordingMode == RECORING_MODE_CONTINUOUS) {
				BRO_toolbar.setRecording(true);
				BRO_ButtonsHandler.recordCurrentPage();
			}
		} 
	},
	
	closePopup : function() {
		var recordButtonPopup = document.getElementById("BRO_recordButtonPopup");
		if(recordButtonPopup) {
			recordButtonPopup.hidePopup();
		}
	},
	
	getLabel : function() {
		var recordButton = document.getElementById("BRO_recordButton");
		
		// The label is located in XBL elements
		var anonymousChildren = document.getAnonymousNodes(recordButton);
		if(anonymousChildren) {
			for (var i = 0; i < anonymousChildren.length; i++) {
				var anonymousChild = anonymousChildren[i];
				if(anonymousChild.localName == "toolbarbutton") {
					var buttonChildren = document.getAnonymousNodes(anonymousChild);
					for (var j = 0; j < buttonChildren.length; j++) {
						var child = buttonChildren[j];
						if(child.localName == "label") {
							return child;
						}
					}
				}	
			}
		}		
	},
	
	fixButtonSize : function() {
		var recordButton = document.getElementById("BRO_recordButton");
		var recordButtonLabel = this.getLabel();		

		if (recordButton && recordButtonLabel) {
			var recordButtonBarMode = recordButton.parentNode.getAttribute('mode');
			if (recordButtonBarMode == 'icons') {
				return;
			}
			var labelWidth = recordButtonLabel.boxObject.width + 5;
			var charWidth = Math.round(labelWidth / recordButtonLabel.value.length);
			var newWidth = labelWidth + (3 * charWidth);
			if (newWidth == 0 || newWidth == null) {
				newWidth = 70;
			}
			recordButtonLabel.style.width = newWidth + "px";
			recordButtonLabel.style.textAlign = "center";
		}
	},	

	refreshRecordButtonLabel : function(isRecording) {
		var recordButtonLabel = this.getLabel();
		if (!recordButtonLabel)
			return;

		// Label showing we are recording
		if (isRecording) {
			recordButtonLabel.value = BRO_locale.getString('record.button.label.pause');
		} else {
			// Label showing this page has already been recorded
			if (BRO_toolbar.recordingMode == RECORING_MODE_ONE_BY_ONE && BRO_toolbar.isCurrentUrlRecorded()) {
				recordButtonLabel.value = BRO_locale.getString('record.button.label.recorded.oneByOne');
			}
			// Label showing we are not recording
			else {
				recordButtonLabel.value = BRO_locale.getString('record.button.label.record');
			}
		}
	},
	
	onClickChangeMode:function(event) {
		this._recordButtonMenuItemCommandFired = true;
		if(BRO_toolbar.getRecordingMode() == RECORING_MODE_ONE_BY_ONE) {
			BRO_toolbar.setRecordingMode(RECORING_MODE_CONTINUOUS);
		}
		else {
			BRO_toolbar.setRecordingMode(RECORING_MODE_ONE_BY_ONE);
		}
	},
	
	refreshModeSelection : function(recordingMode) {
		var recordingMode = (recordingMode) ? recordingMode : BRO_toolbar.getRecordingMode();
		var recordMenuItem = document.getElementById("BRO_modeMenuItem");
		if(recordMenuItem) {		
			if (recordingMode == RECORING_MODE_CONTINUOUS) {
				recordMenuItem.tooltipText = BRO_locale.getString('record.button.tooltip.continuous');
				recordMenuItem.image = "chrome://broceliand/skin/images/pearl/check.png";
				recordMenuItem.label = BRO_locale.getString('record.button.mode.on');
			} 
			else if (recordingMode == RECORING_MODE_ONE_BY_ONE) {
				recordMenuItem.tooltipText = BRO_locale.getString('record.button.tooltip.oneByOne');
				recordMenuItem.image = null;
				recordMenuItem.label = BRO_locale.getString('record.button.mode.off');
			}
		}

		BRO_inButtonController.rebuildTreeListAndBackupSelection();
		BRO_recordButtonController.refreshRecordButtonLabel(BRO_toolbar.isRecording);
	},	

    onClickHelp:function(event) {
        this._recordButtonMenuItemCommandFired = true;
        BRO_toolbar.setFirstUse(false);
        BRO_toolbar.showHelp();
    },
    
    onClickAbout:function(event) {
        this._recordButtonMenuItemCommandFired = true;
        BRO_toolbar.showOptions();
    },	
	
    onClickAddNote:function(event) {
        this._recordButtonMenuItemCommandFired = true;
        BRO_toolbar.showNotePopup();
    }	
}